//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Device/Analyze/Fourier.cpp
//! @brief     Implements Fourier transform function in namespace Analyze.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Device/Analyze/Fourier.h"
#include "Base/Math/FourierTransform.h"
#include "Device/Data/DataUtil.h"
#include "Device/Data/Datafield.h"

namespace {

std::vector<std::vector<double>> FT2DArray(const std::vector<std::vector<double>>& signal)
{
    FourierTransform ft;
    std::vector<std::vector<double>> result;
    ft.fft(signal, result);
    ft.fftshift(result); // low frequency to center of array
    return result;
}

} // namespace

Datafield Analyze::createFFT(const Datafield& data)
{
    auto array_2d = DataUtil::create2DArrayfromDatafield(data);
    auto fft_array_2d = FT2DArray(array_2d);
    return DataUtil::vecvecToDatafield(fft_array_2d);
}
