//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/PlotUtil/RangeUtil.cpp
//! @brief     Implements RangeUtil namespace
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/PlotUtil/RangeUtil.h"
#include "GUI/Model/Axis/AmplitudeAxisItem.h"
#include "GUI/Model/Data/IntensityDataItem.h"
#include "GUI/Model/Data/SpecularDataItem.h"
#include <qcustomplot.h>

namespace {

double commonMin(const QList<AmplitudeAxisItem*>& axes)
{
    double min = +std::numeric_limits<double>::max();
    for (auto axis : axes)
        if (min > axis->min())
            min = axis->min();
    return min;
}

double commonMax(const QList<AmplitudeAxisItem*>& axes)
{
    double max = -std::numeric_limits<double>::max();
    for (auto axis : axes)
        if (max < axis->max())
            max = axis->max();
    return max;
}

QCPRange commonRange(const QList<AmplitudeAxisItem*>& axes)
{
    return QCPRange(commonMin(axes), commonMax(axes));
}

QList<AmplitudeAxisItem*> valueAxesFromSpecularItems(const QList<SpecularDataItem*>& items)
{
    QList<AmplitudeAxisItem*> axes;
    for (auto item : items)
        axes.append(item->yAxisItem());
    return axes;
}

QList<AmplitudeAxisItem*> valueAxesFromIntensityItems(const QList<IntensityDataItem*>& items)
{
    QList<AmplitudeAxisItem*> axes;
    for (auto item : items)
        axes.append(item->zAxisItem());
    return axes;
}

} // namespace

void GUI::View::RangeUtil::setCommonRangeY(QList<SpecularDataItem*> items)
{
    QCPRange range = commonRange(valueAxesFromSpecularItems(items));
    for (auto item : items) {
        item->setLowerY(range.lower);
        item->setUpperY(range.upper);
    }
}

void GUI::View::RangeUtil::setCommonRangeZ(QList<IntensityDataItem*> items)
{
    QCPRange range = commonRange(valueAxesFromIntensityItems(items));
    for (auto item : items) {
        item->setLowerZ(range.lower);
        item->setUpperZ(range.upper);
    }
}
